<?php

namespace Drupal\smallads\Plugin\Field\FieldType;

use Drupal\smallads\Entity\SmalladInterface;
use Drupal\Core\TypedData\OptionsProviderInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\IntegerItem;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Defines the 'scope' smallad field type.
 *
 * @FieldType(
 *   id = "smallad_scope",
 *   label = @Translation("Smallad Scope"),
 *   description = @Translation("How widely this ad can be seen"),
 *   category = @Translation("Smallads"),
 *   default_widget = "options_select"
 * )
 */

class ScopeItem extends IntegerItem implements OptionsProviderInterface{

  /**
   * {@inheritdoc}
   */
  public function getPossibleValues(AccountInterface $account = NULL) {
    return \Drupal::config('smallads.settings')->get('scopes');
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleOptions(AccountInterface $account = NULL) {
    return array_intersect_key(
      smallads_valid_scopes(),
      array_flip($this->getPossibleValues($account))
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableValues(AccountInterface $account = NULL) {
    return $this->getPossibleValues($account);
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableOptions(AccountInterface $account = NULL) {
    return $this->getPossibleOptions($account);
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    return rand($this->getPossibleValues());
  }

  /**
   *
   * Entity field default callback for scope field.
   * The default scope for a smallad is based on the settings for its owner.
   *
   * @todo make this a site setting
   *
   * @param SmalladInterface $smallad
   * @param \Drupal\smallads\Plugin\Field\FieldType\BaseFieldDefinition $definition
   *
   * @return type
   */
  public static function smalladsDefaultScope(SmalladInterface $smallad) {
    return \Drupal::service('user.data')
      ->get('smallads', $smallad->getOwnerId(), 'defaultscope') ?: SmalladInterface::SCOPE_SITE;
  }

}
